@@ -3,10 +3,10 @@ package DBIx::Fast;
 =head1 NAME
  
     DBIx::Fast - DBI fast & easy (another one...)
-
+   
 =cut
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use strict;
 use warnings FATAL => 'all';
@@ -15,9 +15,11 @@ use warnings FATAL => 'all';
  
         use DBIx::Fast;
 
-        my $d = DBIx::Fast->new( db => 'test' , user => 'test' , passwd => 'test');
+        my $DB = DBIx::Fast->new( dsn => 'dbi:MariaDB:database=test:host' , user => 'test' , passwd => 'test');
 
-        my $d = DBIx::Fast->new( db => 'test' , user => 'test' , passwd => 'test',
+        my $DB = DBIx::Fast->new( db => 'test', user => 'test', passwd => 'test', driver => 'MariaDB' );
+
+        my $DB = DBIx::Fast->new( db => 'test', user => 'test', passwd => 'test', driver => 'mysql' );
                                  trace => '1' , profile => '!Statement:!MethodName' );
 
         for (qw(cc co ce)) {
@@ -48,23 +50,51 @@ use warnings FATAL => 'all';
 
 use Carp;
 use Moo;
+
+use DBI;
 use DBIx::Connector;
-use DateTime::Format::MySQL;
 
 has db  => ( is => 'rw' );
+
+=head2 dbd
+ Database type
+=cut
 has dbd => ( is => 'rwp');
 
+=head2 errors
+ Errors in SQL
+=cut
 has errors   => ( is => 'rwp');
+has last_error => ( is => 'rwp' );
 
+=head2 last_sql
+ Return last SQL sentence
+=cut
 has last_sql => ( is => 'rw');
+
+=head2 last_id
+ Return last id
+=cut
 has last_id  => ( is => 'rw');
 
 has sql => ( is => 'rw' );
-
 has p   => ( is => 'rw' );
-
 has results  => ( is => 'rw');
 
+has dsn => ( is => 'rwp' );
+has dbi_args => ( is => 'rwp' );
+
+=head2 now
+  NOW() - Timestamp
+=cut
+sub now {
+    my $self = shift;
+
+    my ($sec, $min, $hour, $mday, $mon , $year) = localtime;
+
+    return sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year + 1900, $mon + 1, $mday, $hour, $min, $sec);
+}
+
 =head2 set_error
     Add error to the array
 =cut
@@ -80,6 +110,7 @@ sub set_error {
     my $Errors = $self->errors;
     push @{$Errors} ,$error;
 
+    $self->_set_last_error(qq{$error->{time} - [$error->{id}] - $error->{error}});
     $self->_set_errors($Errors);
 }
 
@@ -88,33 +119,55 @@ sub set_error {
 =cut
 sub BUILD {
     my ($self,$args) = @_;
-    
-    my $dbi_args = {
-	RaiseError => $args->{Error} // 0,
-	PrintError => $args->{PrintError} // 0,
-	mysql_enable_utf8 => 1,
-	AutoCommit => 1,
+    my $DConf;
+
+    $self->Exception("DBD Driver : Not defined") unless $args->{driver};
+
+    $args->{host} = '127.0.0.1' unless $args->{host};
+
+    $DConf->{args} = {
+	RaiseError => $args->{Error} // 1,
+	PrintError => $args->{PrintError} // 1,
+	AutoCommit => $args->{AutoCommit} // 1,
     };
 
-    my $dsn = $args->{dsn} ? $self->_check_dsn($args->{dsn}) : $self->_make_dsn($args);
+    $DConf->{quote} = $args->{quote} if $args->{quote};
+    
+    $self->_set_dsn($args->{dsn} ? $self->_check_dsn($args->{dsn}) : $self->_make_dsn($args));
+    $self->_set_dbi_args($DConf);
 
+    
     if ( $self->dbd eq 'mysql' ) {
-	$dbi_args->{mysql_enable_utf8} = 1;
+	$DConf->{args}->{mysql_enable_utf8} = 1 if $args->{mysql_enable_utf8};
     }
     
-    $self->db(DBIx::Connector->new( $dsn, 
+    $self->_set_dbi_args($DConf);
+    $self->db(DBIx::Connector->new( $self->dsn, 
 				    $args->{user}, $args->{passwd},
-				    $dbi_args ));
+				    $self->dbi_args->{args} ));
 
     $self->db->mode('ping');
-
+    
+    $self->db->dbh->quote($self->dbi_args->{quote}) if $self->dbi_args->{quote};
+    
     $self->db->dbh->{HandleError} = sub {
 	$self->set_error($DBI::err,$DBI::errstr);
     };
 
     $self->db->dbh->trace($args->{trace},'dbix-fast-trace') if $args->{trace};
 
-    $self->profile($args->{profile}) if $args->{profile};
+    $self->_profile($args->{profile}) if $args->{profile};
+}
+
+=head2 _Driver_dbd
+=cut
+sub _Driver_dbd {
+    my $self = shift;
+    my $dbd  = shift;
+    
+    map { $self->_set_dbd($_) if lc($dbd) eq lc($_) } qw(SQLite Pg MariaDB mysql);
+
+    $self->Exception("Error DBD Driver : $dbd") unless $self->dbd;
 }
 
 =head2 check_dsn
@@ -124,11 +177,11 @@ sub _check_dsn {
     my $self = shift;
     my $dsn  = shift;
 
-    my ($dbi,$server,$db,$host) = split ':', $dsn;
+    my ($dbi,$driver,$db,$host) = split ':', $dsn;
+    
+    $self->_Driver_dbd($driver);
 
-    map { $self->_set_dbd($_) if $server eq $_ } qw(sqlite pg mysql);
-
-    $self->Exception("Server not valid : $server") unless $self->dbd;    
+    return $dsn;
 }
 
 =head2 make_dsn
@@ -137,25 +190,18 @@ sub _check_dsn {
 sub _make_dsn {
     my $self = shift;
     my $args = shift;
+
+    $self->_Driver_dbd($args->{driver});
+
+    return 'dbi:SQLite:dbname='.$args->{db} if $args->{driver} eq 'SQLite';
     
-    if ( $args->{host} && $args->{host} eq 'sqlite' ) {
-	$self->_set_dbd('sqlite');
-	return 'dbi:SQLite:'.$args->{db};
-    } elsif ( $args->{db} eq 'pg' ) {
-	$self->_set_dbd('pg');
-        $args->{host} = '127.0.0.1' unless $args->{host};
-	return 'dbi:pg:database='.$args->{db}.':'.$args->{host};
-    } else {
-	$self->_set_dbd('mysql');
-	$args->{host} = '127.0.0.1' unless $args->{host};
-	return 'dbi:mysql:database='.$args->{db}.':'.$args->{host};
-    }
+    return 'dbi:'.$self->dbd.':database='.$args->{db}.':'.$args->{host};
 }
 
 =head2 profile
     Save profile log : dbix-fast--PID.log
 =cut
-sub profile {
+sub _profile {
     my $self = shift;
     my $stat = shift."/DBI::ProfileDumper/";
 
@@ -175,7 +221,9 @@ sub all {
     my $res = $self->db->dbh->selectall_arrayref($self->sql,
 						 { Slice => {} },@{$self->p});
 
-    $self->results($res) unless $DBI::err;
+    $self->Exception("ERROR all()") if $DBI::err;
+    
+    $self->results($res);
 }
 
 =head2 hash
@@ -192,7 +240,9 @@ sub hash {
 
     my $res = $sth->fetchrow_hashref;
 
-    $self->results($res) unless $DBI::err;
+    $self->Exception("hash()") if $DBI::err;
+    
+    $self->results($res);
 }
 
 =head2 val
@@ -218,12 +268,11 @@ sub array {
 
     $sth->execute(@{$self->p});
 
-    unless ( $DBI::err ) {
-	my @rows = @{ $self->db->dbh->selectcol_arrayref(
-			  $self->sql, undef, @{ $self->p } ) };
+    $self->Exception("array()") if $DBI::err;
 
-	$self->results(\@rows);
-    }
+    my @rows = @{ $self->db->dbh->selectcol_arrayref( $self->sql, undef, @{ $self->p } ) };
+    
+    $self->results(\@rows);
 }
 
 =head2 count
@@ -307,10 +356,9 @@ sub execute {
 	}
     }
 
-    unless ( $DBI::err ) {
-	$self->results($res);
-    }
-
+    $self->Exception("execute()") if $DBI::err;
+    
+    $self->results($res);
 }
 
 =head2 up
@@ -320,9 +368,9 @@ sub up {
     my ($self,$table,$data,$where,$time) = @_;
 
     if ( $time ) {
-	$self->update( $self->TableName(shift) , { sen => $data , where => $where } , time => $time );
+	$self->update( $self->TableName($table) , { sen => $data , where => $where } , time => $time );
     } else {
-	$self->update( $self->TableName(shift) , { sen => $data , where => $where } );
+	$self->update( $self->TableName($table) , { sen => $data , where => $where } );
     }
 }
 
@@ -340,7 +388,6 @@ sub update {
     $skeel->{sen} = $self->extra_args($skeel->{sen},@_) if scalar @_ > 0;
 
     my @p;
-
     my $sql = "UPDATE $table SET ";
 
     for ( keys %{$skeel->{sen}} ) {
@@ -391,12 +438,15 @@ sub insert {
     $self->sql($sql);
     $self->execute_prepare(@p);
 
-    if ( $self->dbd eq 'mysql' ) {
+    if ( $self->dbd eq 'MariaDB' ) {
+	$self->last_id($self->db->dbh->{mariadb_insertid});
+    } elsif ( $self->dbd eq 'mysql' ) {
 	$self->last_id($self->db->dbh->{mysql_insertid});
-    } elsif ( $self->dbd eq 'sqlite' ) {
+    } elsif ( $self->dbd eq 'SQLite' ) {
 	$self->last_id($self->db->dbh->sqlite_last_insert_rowid());
+    } elsif ( $self->dbd eq 'Pg' ) {
+	$self->last_id($self->db->dbh->last_insert_id(undef,undef,$table,undef));
     }
-
 }
 
 =head2 delete
@@ -429,9 +479,8 @@ sub extra_args {
     my $skeel = shift;
     my %args  = @_;
 
-    $skeel->{$args{time}} = DateTime::Format::MySQL->format_datetime(DateTime->now)
-	if $args{time};
-
+    $skeel->{$args{time}} = $self->now() if $args{time};
+    
     return $skeel;
 }
 
@@ -492,7 +541,7 @@ sub TableName {
     my $self  = shift;
     my $table = shift;
 
-    return 1 unless $table =~ /\W/;
+    return $table unless $table =~ /\W/;
 
     $self->Exception("TableName not valid: $table");
 }
@@ -503,8 +552,14 @@ sub TableName {
 sub Exception {
     my $self = shift;
     my $msg  = shift;
+
+    return unless $self->dbi_args->{args}->{PrintError};
     
-    carp "Exception: $msg";
+    my $out  = "Exception: $msg - ";
+
+    $out .= $self->last_error if $self->last_error;
+    
+    carp $out;
 }
 
 =head1 AUTHOR
